/****************************
	blu.c
	
	LU por bloques
	modo de almacenamiento de matrices tipo FORTRAN: por columnas
	
	Octubre - Noviembre 2000
**********************************

	retocada para curso de especialista 2006
	
*******************************/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

void    gm_moler();

#ifdef __DEBUG__
void 	inter_filas();
void	fescribir();
void	esc_int();
void	res_LU();
#endif

/*****************************************************************/
int main(int argc, char **argv) {
	double *a,*b,*c;
	double uno=1., cero=0.;
	int	i,j,k,n,m,lda,ldb,ldc;
	int	info;
	int	*ipiv;
	double	tf, ti, t;
	int	ggg,ini_g,fin_g,inc_g;

	if (argc != 4)
	{
		printf("USO: %s n_inicial n_final factor_mult_n\n", argv[0]);
		exit(0);
	}

	ini_g=atoi(argv[1]);	
	fin_g=atoi(argv[2]);
	inc_g=atoi(argv[3]);
	
	for (ggg=ini_g;ggg<=fin_g;ggg+=inc_g) {
	 	m=ggg;
		k=ggg;
		n=ggg;
		lda=m;
		ldb=k;
		ldc=m;
	
		/* Reservar espacio para matrices */
		b= (double*) malloc(sizeof(double)*k*n);
		a= (double*) malloc(sizeof(double)*m*k);
		c= (double*) malloc(sizeof(double)*m*n);
		ipiv= (int*) malloc(sizeof(int)*(m+k+n));
	
		/* Inicilaizar matrices*/
		gm_moler(a,m,lda);
		gm_moler(b,k,ldb);
		gm_moler(c,k,ldc);

		/* Visualizar contenido matrices */
		#ifdef __DEBUG__
		fescribir(a,lda,m,k,"matriz A original");
		printf("\n lda=%d m=%d k=%d \n");
		#endif
			
		/* calcular LU por bloques llamando directamente a la rutina de LAPACK "dgetrf" */
		ti=clock();
		dgetrf_(&k,&n,b,&ldb,ipiv,&info);
		tf=clock();
		t = (double)(tf-ti)/(double)CLOCKS_PER_SEC;
		if (info)
			printf("[%s] L=dgetrf, N=%.4d, [Error %d]",argv[0],n, info);
		else
  			printf("[%s] L=dgetrf, N=%.4d, S=%lf\n",argv[0],n,t);
		
		/* calcular LU sin utilizar bloques, llamando a la rutina de LAPACK "dgetf2" */ 
		ti=clock();
		dgetf2_(&m,&n,c,&ldc,ipiv,&info);
		tf=clock();
		t = (double)(tf-ti)/(double)CLOCKS_PER_SEC;
		if (info)
			printf("[%s] L=dgetf2, N=%.4d, [Error %d]",argv[0],n, info);
  		else
			printf("[%s] L=dgetf2, N=%.4d, S=%lf\n",argv[0],n,t);
		
		/* visualizar matrices resultado*/
		#ifdef __DEBUG__
                fescribir(a,lda,m,k,"matriz LU ");
                esc_int(ipiv,1,m,1,"ipiv");
                res_LU(a,lda,m,k,ipiv);
                fescribir(a,lda,m,k,"matriz A <= L * U antes de swap filas");
                inter_filas(a,lda,m,k,ipiv);
                fescribir(a,lda,m,k,"matriz A <= L * U despues de swap filas segun 'ipiv'");
		#endif
	
                /*liberar espacio memoria de matrices*/
		free(a);
		free(b);
		free(c);
		free(ipiv);	
	
	}/*for ggg*/
	exit(0);
}/*main*/

/*********************************************************************************************************/
/*
	inter_filas
*/
/*********************************************************************************************************/

#ifdef __DEBUG__
void inter_filas(a,lda,m,k,ipiv)
double *a;
int	lda,m,k,*ipiv;
{
	int i;
	for(i=m-1;i>=0;i--)
	{
		dswap_(&k,&a[ipiv[i]-1],&lda,&a[i],&lda);
	}
}
#endif


/*********************************************************************************************************/
/*
gm_moler:
  genera matriz con aij=min(i,j)-1, i<>j
        aii=i
        */
/*********************************************************************************************************/        
void gm_moler(m,n,ld)
double *m;
int n,ld;
{
  int i,j;
  for(i=0;i<n;i++)
    for(j=i+1;j<n;j++)
    {
      m[i+j*ld]=i-1.;
      m[j+i*ld]=i-1.;
    }
                                    
    for(i=0;i<n;i++)
      m[i+i*ld]=(double) i;
                                          
}

/*********************************************************************************************************/
/*
	fescribir
	al estilo FORTRAN, con las matrices almacenadas por columnas
*/
/*********************************************************************************************************/

#ifdef __DEBUG__
void	fescribir(a,lda,nf,nc,men)
double *a;
int	lda,nf,nc;
char	*men;
{
	int	i,j;

	printf("\n %s : \n",men);
	for(i=0;i<nf;i++)
	{
		for(j=0;j<nc;j++)
		{
			printf("%+6.2lf\t",a[i+j*lda]);
		}
		printf("\n");
	}
}

void	esc_int(a,lda,nf,nc,men)
int *a;
int	lda,nf,nc;
char	*men;
{
	int	i,j;

	printf("\n %s : \n",men);
	for(i=0;i<nf;i++)
	{
		for(j=0;j<nc;j++)
		{
			printf("%+8.3d\t",a[i*lda+j]);
		}
		printf("\n");
	}
}


/*********************************************************************************************************/
/*
	res_LU
		resulelve de una matriz LU compactada, haciendo el producto
		L * U = A
*/
/*********************************************************************************************************/
void	res_LU(a,lda,m,k,ipiv)
double *a;
int	lda,m,k;
int	*ipiv;
{
	double *l,*u;
	int	i,j;
	double	uno=1,cero=0;
	l=(double*)malloc(sizeof(double)*m*k);
	u=(double*)malloc(sizeof(double)*m*k);
	
	for(i=0;i<m;i++)
	{
		for(j=0;j<k;j++)
		{
			if(j==i)
			{
				l[i+j*lda]=1;
				u[i+j*lda]=a[i+j*lda];
			}
			if(j<i)
			{
				l[i+j*lda]=a[i+j*lda];
				u[i+j*lda]=0;
			}
			if(j>i)
			{
				l[i+j*lda]=0;
				u[i+j*lda]=a[i+j*lda];
			}
			
		}/*for j*/
	}/*for i*/
	

	dgemm_("N","N",&m,&k,&k,&uno,l,&lda,u,&lda,&cero,a,&lda);

	free(l);
	free(u);	
}

#endif

