/*
 * mult_mpi.c
 *
 * matriz A : broadcast completa de root a todos los procesos
 * matriz B: gather por bloques de columnas (nada de cclico) a cada proceso
 * matriz C: resultado quedara por bloques de columnas en cada proceso,
 *	se mandaria (scatter) al root
 * 
 *
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include "mpi.h"
 
 int TAMA=1000;
 
 void	inicializa();
 void	escribe();
 void	escribe_2D();
 
 int main(argc,argv)
 int	argc;
 char	*argv[];
 {
  	MPI_Status	status;
  	double		*mi_A,*mi_B,*mi_C;
  	double		*A,*B,*C;
  	int		my_rank,p;
  	int		root=0;
  	double		cero=0.,uno=1.,menos_uno=-1.;
  	int		m,n,k;
  	int		tama_total,tama_local;
  	double		tf,ti;
                                                                                 
 	MPI_Init(&argc,&argv);
        MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
        MPI_Comm_size(MPI_COMM_WORLD,&p);

	if (argc != 2) {
		printf("Falta tamao de matriz\n");
		exit(1);
	}

	TAMA = atoi(argv[1]);
       
       tama_total=TAMA*TAMA;
       tama_local=tama_total/p;
	m=TAMA;
	n=TAMA/p;
	k=TAMA;

      	A=(double*)malloc(sizeof(double)*tama_total);       
      	mi_B=(double*)malloc(sizeof(double)*tama_local);
        mi_C=(double*)malloc(sizeof(double)*tama_local);	
        
       	if(my_rank==0)
        {
        	B=(double*)malloc(sizeof(double)*tama_total);
        	C=(double*)malloc(sizeof(double)*tama_total);
        	inicializa(my_rank,A,tama_total);
       		inicializa(my_rank,B,tama_total);
       	}
       
       
       	//MPI_Barrier(MPI_COMM_WORLD);
        //ti=MPI_Wtime();
                                      
                                      
       
       /*,matriz A replicada a todos*/      
  	MPI_Bcast(A,tama_total,MPI_DOUBLE,root,MPI_COMM_WORLD);  
	/*
  		printf("matriz A recibida por proceso %d, tamanyo=%d:\n",my_rank,tama_total);
	  	escribe_2D(A,TAMA,TAMA);
	*/
  	
  	/*matriz B repartida por bloques de columnas consecutivas*/
	MPI_Scatter(B,tama_local,MPI_DOUBLE,mi_B,tama_local,MPI_DOUBLE,root,MPI_COMM_WORLD);                                                                                                                                                                 
	/*	
		printf("matriz mi_B recibida por proceso %d, tamanyo=%d:\n",my_rank,tama_local);
	  	escribe_2D(mi_B,TAMA,TAMA/p);
  	*/

	/*multiplicacin matrices locales: mi_C =  A * mi_B*/
	MPI_Barrier(MPI_COMM_WORLD);
        ti=MPI_Wtime();
          
		dgemm_("N","N",&m,&n,&k,&uno,A,&m,mi_B,&m,&cero,mi_C,&m);
	
		fprintf(stderr,"matriz tamaño %dx%d  mi_C obtenida por proceso %d:\n",m,n,my_rank);
	  	//escribe_2D(mi_C,m,n);
	MPI_Barrier(MPI_COMM_WORLD);
        tf=MPI_Wtime();
          
	
		
	/*matriz C enviada a cachos hacia el root*/
	MPI_Gather(mi_C,tama_local,MPI_DOUBLE,C,tama_local,MPI_DOUBLE,root,MPI_COMM_WORLD);
	
	
	//MPI_Barrier(MPI_COMM_WORLD);
        //tf=MPI_Wtime();
        
	
       	if(my_rank==0)
        {
        	
        	fprintf(stderr,"\n Tamaño:%d  Tiempo:%lf   Matriz C recibida en root:\n",TAMA,tf-ti);
		printf("[%s] P=%d, N=%d, S=%.6lf\n", argv[0], p, TAMA, tf-ti);
        	//escribe_2D(C,TAMA,TAMA);
        	
        	free(B);
        	free(C);
        }
                        
	free(A);
	free(mi_B);
	free(mi_C);                                                                                                                                                              
	MPI_Finalize();
	exit(0);
 }
 
 void inicializa(my_rank,datos,tamanyo)
 int	my_rank;
 double	*datos;
 int	tamanyo;
 {
 	int	i;
 	for (i=0;i<tamanyo;i++)
 	{
 		datos[i]=my_rank*10000+i;	
 	}
 }
 
void escribe(datos,tamanyo)
 double	*datos;
 int	tamanyo;
 {
 	int	i;
 	for (i=0;i<tamanyo;i++)
 	{
 		fprintf(stderr,"  %lf  ",datos[i]);	
 	}
 }
 
void escribe_2D(datos,num_fil,num_col)
 double	*datos;
 int	num_fil,num_col;
 {
 	int	i,j;
 	for (i=0;i<num_fil;i++)
 	{
 		for (j=0;j<num_col;j++)
 		{
 			fprintf(stderr,"  %lf  ",datos[i+j*num_fil]);	
 		}
 		fprintf(stderr,"\n");
 	}
 	fprintf(stderr,"\n\n");
 }
 
 
