#!/bin/bash

[ -z $IT ] && IT=10            # Repeticiones
[ -z $TMP_DIR ] && TMP_DIR=/dev/shm

OUT=$TMP_DIR/tmpfile.txt
rm $OUT 2>/dev/null

NAME=`pwd | rev | cut -d'/' -f1  | rev`
[ -z "$N" ] && N="10000000 20000000 40000000"

CONFIG="`cat ${NAME}.conf | egrep -v '(^#|^$)'`"
WC=`echo "$CONFIG" | wc -l`
PWD=`pwd`

function write_schema() {
	local t=$1
	local n=$2
	rm esquema* 2>/dev/null
	for i in `seq 1 $WC` ; do
		LINE=`echo "$CONFIG" | head -n $i | tail -n 1`
		NODE=`echo "$LINE" | cut -d' ' -f1`
		HOST=`echo "$LINE" | cut -d' ' -f2`
		BITS=`echo "$LINE" | cut -d' ' -f3`
		[ -f esquema$(($i-1)) ] && cp esquema$(($i-1)) esquema$i
	        echo "$NODE ${PWD}/${NAME}${BITS} $t $n" >> esquema$i
	done
}

for i in `seq 1 $WC` ; do
	LINE=`echo "$CONFIG" | head -n $i | tail -n 1`
	NODE=`echo "$LINE" | cut -d' ' -f1`
	HOST=`echo "$LINE" | cut -d' ' -f2`
	BITS=`echo "$LINE" | cut -d' ' -f3`
	echo "#### Nodo $HOST ###"
	echo "Haciendo ping..."
	ping -c 1 $HOST
	echo "Compilando..."
	ssh $HOST "cd $PWD ; mpicc $NAME.c -o ${NAME}${BITS} -O2 -march=native"
done

echo '#### Lanzando las pruebas ####'

rm *.resultados 2>/dev/null

for n in $N ; do
   for t in s i b g c; do
      for s in `seq 1 $WC` ; do

	rm $OUT 2>/dev/null
	write_schema $t $n
	echo -n "Ejecutando con N=$n, Prueba=$t, Schema=$s " 
	for i in `seq 1 $IT` ; do
		echo -n "."
		mpirun esquema$s 2>/dev/null >>$OUT
	done
	echo

	PRIMIT=`cat $OUT | tr ']' '[' | cut -d'[' -f2 | uniq | tr '/' '-'`
	MEDIA=$(echo scale=7\; \(`cat $OUT | sed "s|.*, S=\(.*\)|\1|g" | tr "\n" "+"`0\) / $IT | bc -l)
	MEDIA=`echo $MEDIA | sed "s|^\.|0.|g"`
	echo `cat $OUT | head -n 1 | cut -d, -f-2`, S=$MEDIA >> ${NAME}-${PRIMIT}.resultados

      done
   done
done

rm $OUT
