/*
c
c Introducción a la Programación Paralela
c Capítulo 6
c Paralelismo Síncrono: relajación de Jacobi en OpenMP con paralelismo implícito
c 
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>
#include <omp.h>

void escribir(double *m, int t){
  int i,j;

  for (i = 0; i < t; i++)
  {
    for(j=0;j<t;j++)
      fprintf(stderr,"%.4lf ",m[i*t+j]);
    fprintf(stderr,"\n");
  }
  fprintf(stderr,"\n");
}

void copia(double *a, double *b,int t){
  int i;

  for (i = 0; i < t*t; i++)
    b[i]=a[i];
}


////////////////////////////////////////////////////////77
//			jacobi
////////////////////////////////////////////////////////////
void jacobi(double *a,double *b,int t)
{
  int i,j;
  double s=1.,tol=0.01;

//  while(s>tol)
// dejamos una única iteracion para tomar tiempos más controlables	
  {
    s=0.;
	#pragma omp parallel for private(j)
    for(i=1;i<t-1;i++)
      for(j=1;j<t-1;j++)
        b[i*t+j]=(a[(i-1)*t+j]+a[(i+1)*t+j]+a[i*t+j-1]+a[i*t+j+1])/4;

	#pragma omp parallel for private(j)
    for(i=1;i<t-1;i++)
      for(j=1;j<t-1;j++)
        a[i*t+j]=(b[(i-1)*t+j]+b[(i+1)*t+j]+b[i*t+j-1]+b[i*t+j+1])/4;

	#pragma omp parallel for private(j) reduction(+:s)
    for(i=1;i<t-1;i++)
      for(j=1;j<t-1;j++)
        s+=fabs(a[i*t+j]-b[i*t+j]);
  }
}

void initializealea(double *m, int t){
  int i;

  for (i = 0; i < t*t; i++)
    m[i] = 10.*((double)rand()/RAND_MAX);
}



///////////////////////////////////////////////////////////////////////////////////
/*
c
c     mseconds - returns elapsed microseconds since Jan 1st, 1970.
c
*/
double mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000000. + t.tv_usec;
}




/////////////////////////////////////////////////////////////////////
int main(int argc,char *argv[]) {
  double start,fin,tiempo;
  double *a,*b;
  int THREADS,t,p;

  if (argc < 3) {
  	printf("\n\n Uso:  %s N P \n\n",argv[0]);
	exit(1);
  }

  t=atoi(argv[1]);
  THREADS=atoi(argv[2]);

  omp_set_num_threads(THREADS);

  a = (double *) malloc(sizeof(double)*(t+1)*(t+1));
  b = (double *) malloc(sizeof(double)*(t+1)*(t+1));
  initializealea(a,t+1);
  escribir(a,t+1);
  copia(a,b,t+1);

  start=mseconds();

  jacobi(a,b,t+1);

  fin=mseconds();
  tiempo=(fin-start)/1000000.;

  if (tiempo == 0.)
  	printf("[%s] P=%d, N=%d, S=0 - No hay suficiente precision\n", argv[0], THREADS, t);
  else {
	printf("[%s] P=%d, N=%d, S=%.6lf\n", argv[0], THREADS, t, tiempo);
	escribir(a,t+1);
  }

  free(a);
  free(b);
}

