/*
	A * Bt --> C

para acceder por filas a A y a B buscando localidad espacial

*/

#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
int THREADS;



/************************************************
	mm

	A mxk  *  Bt kxn  -->  C mxn
****************************************************/
void mm(double *a,double *b,double *c,int m, int n, int k) 
{
  int i,j,iam,nprocs,kk;
  double s;

  //printf("Multiplicando: A %dx%d   *   B %dx%d   -->  C %dx%d  \n",m,k,k,n,m,n); 
  #pragma omp parallel private(iam,nprocs) 
  {
    nprocs=omp_get_num_threads();
    iam=omp_get_thread_num();

    #pragma omp master
      THREADS=nprocs;



    #pragma omp for private(i,s,j,kk) schedule(static)
    for (i=0;i<m;i++) 
	{
		//printf(" thread: %d , i=%d \n",iam,i);

		for (j=0;j<n;j++)
		{

			s=0.;	
			for	(kk=0;kk<k;kk++)
			{

				s+=a[i*k+kk]*b[j*k+kk];
			}		
			c[i*n+j]=s;
		}//for j
    }//for i

  }//parallel private

}



//////////////////////////////////////////////////////////
void initialize(double *m,int t) {
  int i;
  for (i = 0; i < t; i++)
    m[i] = (double)(i);
}

void initializealea(double *m, int t) {
  int i;
  for (i = 0; i < t; i++)
    m[i] = (double)rand()/RAND_MAX;
}

void escribir(double *m, int fm,int cm,int ldm) {
  int i,j;
  for (i = 0; i < fm; i++) {
    for(j=0;j<cm;j++)
      fprintf(stderr, "%.4lf ",m[i*ldm+j]);
    fprintf(stderr, "\n");
  }
}



////////////////////////////////////////////////////////////////////////7
int main(int argc,char *argv[]) {
  int t,iN,fN,incN,i;
  double start,fin,tiempo,Mflops;
  double *a,*b,*c;
	
	if (argc<4)
	{
		printf(" Uso:  %s inicioN finN incrementoN \n\n",argv[0]);
		exit(1);
	}
  iN=atoi(argv[1]);
  fN=atoi(argv[2]);
  incN=atoi(argv[3]);
  for(t=iN;t<=fN;t+=incN) {
    a = (double *) malloc(sizeof(double)*t*t);
    b = (double *) malloc(sizeof(double)*t*t);
    c = (double *) malloc(sizeof(double)*t*t);
    initializealea(a,t*t);
    initializealea(b,t*t);
    initializealea(c,t*t);

    start=omp_get_wtime();
    mm(a,b,c,t,t,t);
    fin=omp_get_wtime();

    tiempo=fin-start;
    if(tiempo==0.) {
      printf("No hay suficiente precision\n");
    }
    else {
      Mflops=((2.*t*t)/tiempo)/1000000.;
      printf("[%s] P=%d, N=%d, S=%.6lf\n", argv[0], THREADS, t, tiempo);
      //printf("  threads %d, tamano %d segundos: %.6lf\n",THREADS,t,tiempo);
      //printf("  threads %d, tamano %d segundos: %.6lf, Mflops: %.6lf, Mflops por thread: %.6lf\n\n\n",THREADS,t,tiempo,Mflops,Mflops/THREADS);
    }

	fprintf(stderr," \n matriz A \n");
	for(i=0;i<t*t;i++)
	{
		fprintf(stderr,"%lf ",a[i]);
	}

	fprintf(stderr," \n matriz B \n");
	for(i=0;i<t*t;i++)
	{
		fprintf(stderr,"%lf ",b[i]);
	}

	fprintf(stderr," \n matriz C \n");
	for(i=0;i<t*t;i++)
	{
		fprintf(stderr,"%lf ",c[i]);
	}

    free(a);
    free(b);
    free(c);
  }
}
