#!/bin/bash

[ -z $IT ] && IT=10		# Repeticiones
[ -z $P_MAX ] && P_MAX=10	# Threads maximos
[ -z $TMP_DIR ] && TMP_DIR=/dev/shm
[ -z "$N" ] && N="1 50 100 1000 5000"

TMP=$TMP_DIR/tmpfile.txt
NAME=`pwd | rev | cut -d'/' -f1  | rev`

PATH=.:$PATH

for o in -O2 -O3 -Os ; do 

  echo "Compilando"
  gcc -march=native $o --openmp -fomit-frame-pointer ${NAME}.c -o ${NAME}_gcc$o
  icc -m64 $o -openmp -fomit-frame-pointer ${NAME}.c -o ${NAME}_icc$o

  echo "Ejecutando"
  for c in gcc icc ; do
	echo "------- Compilador $c -----------------------------"
	rm ${NAME}_$c$o.resultados 2>/dev/null
	for n in $N ; do
		for p in `seq 1 $P_MAX` ; do
			echo "Ejecutando $IT veces con P=$p, N=$n"
			rm $TMP 2>/dev/null
			for i in `seq 1 $IT` ; do 
				#Parametros: tamBucle=10, numTareas=$n, numThreads=$p, computacion=1000
				${NAME}_$c$o 10 $n $p 1000 2>/dev/null >>$TMP
			done
			MEDIA=$(echo scale=7\; \(`cat $TMP | sed "s|.*, S=\(.*\)|\1|g" | tr "\n" "+"`0\) / $IT | bc -l)
			MEDIA=`echo $MEDIA | sed "s|^\.|0.|g"`
			echo `cat $TMP | head -n 1 | cut -d, -f-2`, S=$MEDIA >> ${NAME}_$c$o.resultados
		done
	done
  done

done

rm $TMP
