#!/bin/bash

[ -z $TMP_DIR ] && TMP_DIR=/dev/shm
NAME=`pwd | rev | cut -d'/' -f1  | rev`
[ -z "$N" ] && N="1 50 100 1000 5000"

if [ "x$LOG_SCALE" == "x1" ] ; then
	LOG_FILE='_logY'
	LOG_STR='set logscale y'
	SET_PLOT_STR=$LOG_STR
else
	SET_PLOT_STR='set ytics 0.0001'
fi

for o in -O2 -O3 -Os ; do
for c in gcc icc ; do
	
	# Un grafico para cada tamagno en un compilador determinado
	F=${NAME}_$c$o.resultados
	for n in $N ; do
		OUT="$TMP_DIR/data_${c}${o}_$n"
		cat $F | grep " N=$n," | sed "s|.* P=\(.*\), N=$n, S=\(.*\)|\1 \2|g" > $OUT
		cat <<__EOF__ | gnuplot
set terminal pdf
set output "${NAME}_${c}${o}_N$n$LOG_FILE.pdf"
set xlabel "Threads"
set ylabel "Segundos"
set title "$NAME"
set xtics 1
$LOG_STR
plot "$OUT" using 1:2 with linespoints title "$c" 1
__EOF__
	done

	PLOT=''
	COLOR=1
	for n in $N ; do
		PLOT="$PLOT, \"$TMP_DIR/data_${c}${o}_${n}\" using 1:2 with linespoints title \"N=${n}\" $COLOR"
		COLOR=$(($COLOR+1))
	done
	PLOT=`echo $PLOT | cut -c3-`

	# Graficos con todos los tamagnos en un compilador determinado
	cat <<__EOF__ | gnuplot
set terminal pdf
set output "${NAME}_$c$LOG_FILE.pdf"
set key right bottom
set xlabel "Threads"
set ylabel "Segundos"
set title "$NAME"
set xtics 1
$SET_PLOT_STR
plot $PLOT
__EOF__
done
done

for o in -O2 -O3 -Os ; do
# Graficos comparando gcc y icc para cada tamagno
for n in $N ; do
	cat <<__EOF__ | gnuplot
set terminal pdf
set output "${NAME}_gcc_icc_N$n$o$LOG_FILE.pdf"
set key right bottom
set xlabel "Threads"
set ylabel "Segundos"
set title "${NAME}, N=$n"
set xtics 1
$LOG_STR
plot "$TMP_DIR/data_gcc${o}_$n" using 1:2 with linespoints title "gcc" 1 ,\
     "$TMP_DIR/data_icc${o}_$n" using 1:2 with linespoints title "icc" 3
__EOF__
done
done

rm $TMP_DIR/data_*
