#!/bin/bash

[ -z $IT ] && IT=10		# Repeticiones
[ -z $P_MAX ] && P_MAX=10	# Threads maximos
[ -z $TMP_DIR ] && TMP_DIR=/dev/shm
[ -z "$N" ] && N="1 50 100 1000 5000"

TMP=$TMP_DIR/tmpfile.txt
NAME=`pwd | rev | cut -d'/' -f1  | rev`

PATH=.:$PATH

for p in `seq 1 $P_MAX` ; do
  echo "Creando fichero C con $p secciones"
  ./make_sections_c.sh $p
done

for o in -O2 -O3 -Os ; do 

  echo "Compilando con gcc/icc y $o"
  for p in `seq 1 $P_MAX` ; do
  	gcc -march=native $o --openmp -fomit-frame-pointer ${NAME}_P${p}.c -o ${NAME}_P${p}_gcc$o
  	icc -m64 $o -openmp -fomit-frame-pointer ${NAME}_P${p}.c -o ${NAME}_P${p}_icc$o
  done

  echo "Ejecutando con optimizaciones $o"
  for c in gcc icc ; do
	echo "------- Lanzando ejecutables de $c ---------"
	rm ${NAME}_$c$o.resultados 2>/dev/null
	for n in $N ; do
		for p in `seq 1 $P_MAX` ; do
			echo "Ejecutando $IT veces con P=$p, N=$n"
			rm $TMP 2>/dev/null
			for i in `seq 1 $IT` ; do 
				# tamBucle=$n, computacion=50
				${NAME}_P${p}_$c$o $n 50 >>$TMP
			done
			MEDIA=$(echo scale=7\; \(`cat $TMP | sed "s|.*, S=\(.*\)|\1|g" | tr "\n" "+"`0\) / $IT | bc -l)
			MEDIA=`echo $MEDIA | sed "s|^\.|0.|g"`
			echo `cat $TMP | head -n 1 | cut -d, -f-2`, S=$MEDIA >> ${NAME}_$c$o.resultados
		done
	done
  done

done

rm $TMP
